# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.conversation_initiation_source import ConversationInitiationSource
from ...types.conversation_product import ConversationProduct
from ...types.conversation_signed_url_response_model import ConversationSignedUrlResponseModel
from ...types.evaluation_success_result import EvaluationSuccessResult
from ...types.get_conversation_response_model import GetConversationResponseModel
from ...types.get_conversations_page_response_model import GetConversationsPageResponseModel
from ...types.get_sip_log_messages_response import GetSipLogMessagesResponse
from ...types.token_response_model import TokenResponseModel
from .types.conversations_get_request_format import ConversationsGetRequestFormat
from .types.conversations_list_request_exclude_statuses_item import ConversationsListRequestExcludeStatusesItem
from .types.conversations_list_request_summary_mode import ConversationsListRequestSummaryMode


class RawConversationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_signed_url(
        self,
        *,
        agent_id: str,
        include_conversation_id: typing.Optional[bool] = None,
        branch_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ConversationSignedUrlResponseModel]:
        """
        Get a signed url to start a conversation with an agent with an agent that requires authorization

        Parameters
        ----------
        agent_id : str
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        include_conversation_id : typing.Optional[bool]
            Whether to include a conversation_id with the response. If included, the conversation_signature cannot be used again.

        branch_id : typing.Optional[str]
            The ID of the branch to use

        environment : typing.Optional[str]
            The environment to use for resolving environment variables (e.g. 'production', 'staging'). Defaults to 'production'.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationSignedUrlResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/conversation/get-signed-url",
            method="GET",
            params={
                "agent_id": agent_id,
                "include_conversation_id": include_conversation_id,
                "branch_id": branch_id,
                "environment": environment,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationSignedUrlResponseModel,
                    construct_type(
                        type_=ConversationSignedUrlResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_webrtc_token(
        self,
        *,
        agent_id: str,
        participant_name: typing.Optional[str] = None,
        branch_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TokenResponseModel]:
        """
        Get a WebRTC session token for real-time communication.

        Parameters
        ----------
        agent_id : str
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        participant_name : typing.Optional[str]
            Optional custom participant name. If not provided, user ID will be used

        branch_id : typing.Optional[str]
            The ID of the branch to use

        environment : typing.Optional[str]
            The environment to use for resolving environment variables (e.g. 'production', 'staging'). Defaults to 'production'.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TokenResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/conversation/token",
            method="GET",
            params={
                "agent_id": agent_id,
                "participant_name": participant_name,
                "branch_id": branch_id,
                "environment": environment,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TokenResponseModel,
                    construct_type(
                        type_=TokenResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        agent_id: typing.Optional[str] = None,
        call_successful: typing.Optional[EvaluationSuccessResult] = None,
        call_start_before_unix: typing.Optional[int] = None,
        call_start_after_unix: typing.Optional[int] = None,
        call_duration_min_secs: typing.Optional[int] = None,
        call_duration_max_secs: typing.Optional[int] = None,
        rating_max: typing.Optional[int] = None,
        rating_min: typing.Optional[int] = None,
        has_feedback_comment: typing.Optional[bool] = None,
        user_id: typing.Optional[str] = None,
        evaluation_params: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        data_collection_params: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tool_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tool_names_successful: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tool_names_errored: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        main_languages: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        page_size: typing.Optional[int] = None,
        summary_mode: typing.Optional[ConversationsListRequestSummaryMode] = None,
        search: typing.Optional[str] = None,
        conversation_initiation_source: typing.Optional[ConversationInitiationSource] = None,
        text_only: typing.Optional[bool] = None,
        conversation_product_type: typing.Optional[ConversationProduct] = None,
        branch_id: typing.Optional[str] = None,
        topic_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        exclude_statuses: typing.Optional[
            typing.Union[
                ConversationsListRequestExcludeStatusesItem,
                typing.Sequence[ConversationsListRequestExcludeStatusesItem],
            ]
        ] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        workflow_node_entered_id: typing.Optional[str] = None,
        termination_reasons: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetConversationsPageResponseModel]:
        """
        Get all conversations of agents that user owns. With option to restrict to a specific agent.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        agent_id : typing.Optional[str]
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        call_successful : typing.Optional[EvaluationSuccessResult]
            The result of the success evaluation

        call_start_before_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations up to this start date.

        call_start_after_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations after to this start date.

        call_duration_min_secs : typing.Optional[int]
            Minimum call duration in seconds.

        call_duration_max_secs : typing.Optional[int]
            Maximum call duration in seconds.

        rating_max : typing.Optional[int]
            Maximum overall rating (1-5).

        rating_min : typing.Optional[int]
            Minimum overall rating (1-5).

        has_feedback_comment : typing.Optional[bool]
            Filter conversations with user feedback comments.

        user_id : typing.Optional[str]
            Filter conversations by the user ID who initiated them.

        evaluation_params : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Evaluation filters. Repeat param. Format: criteria_id:result. Example: eval=value_framing:success

        data_collection_params : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Data collection filters. Repeat param. Format: id:op:value where op is one of eq|neq|gt|gte|lt|lte|in|exists|missing. For in, pipe-delimit values.

        tool_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by tool names used during the call.

        tool_names_successful : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by tool names that had successful calls.

        tool_names_errored : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by tool names that had errored calls.

        main_languages : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by detected main language (language code).

        page_size : typing.Optional[int]
            How many conversations to return at maximum. Can not exceed 100, defaults to 30.

        summary_mode : typing.Optional[ConversationsListRequestSummaryMode]
            Whether to include transcript summaries in the response.

        search : typing.Optional[str]
            Full-text or fuzzy search over transcript messages

        conversation_initiation_source : typing.Optional[ConversationInitiationSource]

        text_only : typing.Optional[bool]

        conversation_product_type : typing.Optional[ConversationProduct]
            Restrict results to a single conversation product surface.

        branch_id : typing.Optional[str]
            Filter conversations by branch ID.

        topic_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by topic IDs assigned during topic discovery.

        exclude_statuses : typing.Optional[typing.Union[ConversationsListRequestExcludeStatusesItem, typing.Sequence[ConversationsListRequestExcludeStatusesItem]]]
            Exclude conversations with the given statuses. Useful for hiding in-progress / processing conversations from list views.

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by conversation tag IDs assigned via the conversation-tags endpoints.

        workflow_node_entered_id : typing.Optional[str]
            Filter conversations to only those that entered the given node.

        termination_reasons : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by their stored termination_reason (metadata.termination_reason). Repeat param to match any of several.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetConversationsPageResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/conversations",
            method="GET",
            params={
                "cursor": cursor,
                "agent_id": agent_id,
                "call_successful": call_successful,
                "call_start_before_unix": call_start_before_unix,
                "call_start_after_unix": call_start_after_unix,
                "call_duration_min_secs": call_duration_min_secs,
                "call_duration_max_secs": call_duration_max_secs,
                "rating_max": rating_max,
                "rating_min": rating_min,
                "has_feedback_comment": has_feedback_comment,
                "user_id": user_id,
                "evaluation_params": evaluation_params,
                "data_collection_params": data_collection_params,
                "tool_names": tool_names,
                "tool_names_successful": tool_names_successful,
                "tool_names_errored": tool_names_errored,
                "main_languages": main_languages,
                "page_size": page_size,
                "summary_mode": summary_mode,
                "search": search,
                "conversation_initiation_source": conversation_initiation_source,
                "text_only": text_only,
                "conversation_product_type": conversation_product_type,
                "branch_id": branch_id,
                "topic_ids": topic_ids,
                "exclude_statuses": exclude_statuses,
                "tag_ids": tag_ids,
                "workflow_node_entered_id": workflow_node_entered_id,
                "termination_reasons": termination_reasons,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationsPageResponseModel,
                    construct_type(
                        type_=GetConversationsPageResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        conversation_id: str,
        *,
        format: typing.Optional[ConversationsGetRequestFormat] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetConversationResponseModel]:
        """
        Get the details of a particular conversation

        Parameters
        ----------
        conversation_id : str
            The id of the conversation you're taking the action on.

        format : typing.Optional[ConversationsGetRequestFormat]
            Response format. Defaults to 'json'. Set to 'opentelemetry' for an OTLP-compatible trace payload using the same structure as the post-call webhook.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetConversationResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/conversations/{jsonable_encoder(conversation_id)}",
            method="GET",
            params={
                "format": format,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationResponseModel,
                    construct_type(
                        type_=GetConversationResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, conversation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Any]:
        """
        Delete a particular conversation

        Parameters
        ----------
        conversation_id : str
            The id of the conversation you're taking the action on.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/conversations/{jsonable_encoder(conversation_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_sip_messages(
        self,
        conversation_id: str,
        *,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetSipLogMessagesResponse]:
        """
        Get SIP messages associated with a conversation's phone call

        Parameters
        ----------
        conversation_id : str
            The id of the conversation you're taking the action on.

        page_size : typing.Optional[int]

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSipLogMessagesResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/conversations/{jsonable_encoder(conversation_id)}/sip-messages",
            method="GET",
            params={
                "page_size": page_size,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSipLogMessagesResponse,
                    construct_type(
                        type_=GetSipLogMessagesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawConversationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_signed_url(
        self,
        *,
        agent_id: str,
        include_conversation_id: typing.Optional[bool] = None,
        branch_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ConversationSignedUrlResponseModel]:
        """
        Get a signed url to start a conversation with an agent with an agent that requires authorization

        Parameters
        ----------
        agent_id : str
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        include_conversation_id : typing.Optional[bool]
            Whether to include a conversation_id with the response. If included, the conversation_signature cannot be used again.

        branch_id : typing.Optional[str]
            The ID of the branch to use

        environment : typing.Optional[str]
            The environment to use for resolving environment variables (e.g. 'production', 'staging'). Defaults to 'production'.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationSignedUrlResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/conversation/get-signed-url",
            method="GET",
            params={
                "agent_id": agent_id,
                "include_conversation_id": include_conversation_id,
                "branch_id": branch_id,
                "environment": environment,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationSignedUrlResponseModel,
                    construct_type(
                        type_=ConversationSignedUrlResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_webrtc_token(
        self,
        *,
        agent_id: str,
        participant_name: typing.Optional[str] = None,
        branch_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TokenResponseModel]:
        """
        Get a WebRTC session token for real-time communication.

        Parameters
        ----------
        agent_id : str
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        participant_name : typing.Optional[str]
            Optional custom participant name. If not provided, user ID will be used

        branch_id : typing.Optional[str]
            The ID of the branch to use

        environment : typing.Optional[str]
            The environment to use for resolving environment variables (e.g. 'production', 'staging'). Defaults to 'production'.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TokenResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/conversation/token",
            method="GET",
            params={
                "agent_id": agent_id,
                "participant_name": participant_name,
                "branch_id": branch_id,
                "environment": environment,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TokenResponseModel,
                    construct_type(
                        type_=TokenResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        agent_id: typing.Optional[str] = None,
        call_successful: typing.Optional[EvaluationSuccessResult] = None,
        call_start_before_unix: typing.Optional[int] = None,
        call_start_after_unix: typing.Optional[int] = None,
        call_duration_min_secs: typing.Optional[int] = None,
        call_duration_max_secs: typing.Optional[int] = None,
        rating_max: typing.Optional[int] = None,
        rating_min: typing.Optional[int] = None,
        has_feedback_comment: typing.Optional[bool] = None,
        user_id: typing.Optional[str] = None,
        evaluation_params: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        data_collection_params: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tool_names: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tool_names_successful: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tool_names_errored: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        main_languages: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        page_size: typing.Optional[int] = None,
        summary_mode: typing.Optional[ConversationsListRequestSummaryMode] = None,
        search: typing.Optional[str] = None,
        conversation_initiation_source: typing.Optional[ConversationInitiationSource] = None,
        text_only: typing.Optional[bool] = None,
        conversation_product_type: typing.Optional[ConversationProduct] = None,
        branch_id: typing.Optional[str] = None,
        topic_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        exclude_statuses: typing.Optional[
            typing.Union[
                ConversationsListRequestExcludeStatusesItem,
                typing.Sequence[ConversationsListRequestExcludeStatusesItem],
            ]
        ] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        workflow_node_entered_id: typing.Optional[str] = None,
        termination_reasons: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetConversationsPageResponseModel]:
        """
        Get all conversations of agents that user owns. With option to restrict to a specific agent.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        agent_id : typing.Optional[str]
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        call_successful : typing.Optional[EvaluationSuccessResult]
            The result of the success evaluation

        call_start_before_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations up to this start date.

        call_start_after_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations after to this start date.

        call_duration_min_secs : typing.Optional[int]
            Minimum call duration in seconds.

        call_duration_max_secs : typing.Optional[int]
            Maximum call duration in seconds.

        rating_max : typing.Optional[int]
            Maximum overall rating (1-5).

        rating_min : typing.Optional[int]
            Minimum overall rating (1-5).

        has_feedback_comment : typing.Optional[bool]
            Filter conversations with user feedback comments.

        user_id : typing.Optional[str]
            Filter conversations by the user ID who initiated them.

        evaluation_params : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Evaluation filters. Repeat param. Format: criteria_id:result. Example: eval=value_framing:success

        data_collection_params : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Data collection filters. Repeat param. Format: id:op:value where op is one of eq|neq|gt|gte|lt|lte|in|exists|missing. For in, pipe-delimit values.

        tool_names : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by tool names used during the call.

        tool_names_successful : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by tool names that had successful calls.

        tool_names_errored : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by tool names that had errored calls.

        main_languages : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by detected main language (language code).

        page_size : typing.Optional[int]
            How many conversations to return at maximum. Can not exceed 100, defaults to 30.

        summary_mode : typing.Optional[ConversationsListRequestSummaryMode]
            Whether to include transcript summaries in the response.

        search : typing.Optional[str]
            Full-text or fuzzy search over transcript messages

        conversation_initiation_source : typing.Optional[ConversationInitiationSource]

        text_only : typing.Optional[bool]

        conversation_product_type : typing.Optional[ConversationProduct]
            Restrict results to a single conversation product surface.

        branch_id : typing.Optional[str]
            Filter conversations by branch ID.

        topic_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by topic IDs assigned during topic discovery.

        exclude_statuses : typing.Optional[typing.Union[ConversationsListRequestExcludeStatusesItem, typing.Sequence[ConversationsListRequestExcludeStatusesItem]]]
            Exclude conversations with the given statuses. Useful for hiding in-progress / processing conversations from list views.

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by conversation tag IDs assigned via the conversation-tags endpoints.

        workflow_node_entered_id : typing.Optional[str]
            Filter conversations to only those that entered the given node.

        termination_reasons : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter conversations by their stored termination_reason (metadata.termination_reason). Repeat param to match any of several.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetConversationsPageResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/conversations",
            method="GET",
            params={
                "cursor": cursor,
                "agent_id": agent_id,
                "call_successful": call_successful,
                "call_start_before_unix": call_start_before_unix,
                "call_start_after_unix": call_start_after_unix,
                "call_duration_min_secs": call_duration_min_secs,
                "call_duration_max_secs": call_duration_max_secs,
                "rating_max": rating_max,
                "rating_min": rating_min,
                "has_feedback_comment": has_feedback_comment,
                "user_id": user_id,
                "evaluation_params": evaluation_params,
                "data_collection_params": data_collection_params,
                "tool_names": tool_names,
                "tool_names_successful": tool_names_successful,
                "tool_names_errored": tool_names_errored,
                "main_languages": main_languages,
                "page_size": page_size,
                "summary_mode": summary_mode,
                "search": search,
                "conversation_initiation_source": conversation_initiation_source,
                "text_only": text_only,
                "conversation_product_type": conversation_product_type,
                "branch_id": branch_id,
                "topic_ids": topic_ids,
                "exclude_statuses": exclude_statuses,
                "tag_ids": tag_ids,
                "workflow_node_entered_id": workflow_node_entered_id,
                "termination_reasons": termination_reasons,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationsPageResponseModel,
                    construct_type(
                        type_=GetConversationsPageResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        conversation_id: str,
        *,
        format: typing.Optional[ConversationsGetRequestFormat] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetConversationResponseModel]:
        """
        Get the details of a particular conversation

        Parameters
        ----------
        conversation_id : str
            The id of the conversation you're taking the action on.

        format : typing.Optional[ConversationsGetRequestFormat]
            Response format. Defaults to 'json'. Set to 'opentelemetry' for an OTLP-compatible trace payload using the same structure as the post-call webhook.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetConversationResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/conversations/{jsonable_encoder(conversation_id)}",
            method="GET",
            params={
                "format": format,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetConversationResponseModel,
                    construct_type(
                        type_=GetConversationResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, conversation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Delete a particular conversation

        Parameters
        ----------
        conversation_id : str
            The id of the conversation you're taking the action on.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/conversations/{jsonable_encoder(conversation_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_sip_messages(
        self,
        conversation_id: str,
        *,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetSipLogMessagesResponse]:
        """
        Get SIP messages associated with a conversation's phone call

        Parameters
        ----------
        conversation_id : str
            The id of the conversation you're taking the action on.

        page_size : typing.Optional[int]

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSipLogMessagesResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/conversations/{jsonable_encoder(conversation_id)}/sip-messages",
            method="GET",
            params={
                "page_size": page_size,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSipLogMessagesResponse,
                    construct_type(
                        type_=GetSipLogMessagesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .conversation_history_transcript_api_integration_webhook_tools_result_common_model_input import (
    ConversationHistoryTranscriptApiIntegrationWebhookToolsResultCommonModelInput,
)
from .conversation_history_transcript_other_tools_result_common_model import (
    ConversationHistoryTranscriptOtherToolsResultCommonModel,
)
from .conversation_history_transcript_system_tool_result_common_model_input import (
    ConversationHistoryTranscriptSystemToolResultCommonModelInput,
)

if typing.TYPE_CHECKING:
    from .conversation_history_transcript_workflow_tools_result_common_model_input import (
        ConversationHistoryTranscriptWorkflowToolsResultCommonModelInput,
    )
WorkflowToolNestedToolsStepModelInputResultsItem = typing.Union[
    ConversationHistoryTranscriptOtherToolsResultCommonModel,
    ConversationHistoryTranscriptSystemToolResultCommonModelInput,
    ConversationHistoryTranscriptApiIntegrationWebhookToolsResultCommonModelInput,
    "ConversationHistoryTranscriptWorkflowToolsResultCommonModelInput",
]



# This file was auto-generated by Fern from our API Definition.

import typing

CreatePrivateKeyJwtRequestAlgorithm = typing.Union[
    typing.Literal["HS256", "HS384", "HS512", "RS256", "RS384", "RS512"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .....types.embedding_model_enum import EmbeddingModelEnum
from .....types.knowledge_base_document_chunk_response_model import KnowledgeBaseDocumentChunkResponseModel
from .raw_client import AsyncRawChunkClient, RawChunkClient


class ChunkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawChunkClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawChunkClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawChunkClient
        """
        return self._raw_client

    def get(
        self,
        documentation_id: str,
        chunk_id: str,
        *,
        embedding_model: typing.Optional[EmbeddingModelEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> KnowledgeBaseDocumentChunkResponseModel:
        """
        Get details about a specific documentation part used by RAG.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        chunk_id : str
            The id of a document RAG chunk from the knowledge base.

        embedding_model : typing.Optional[EmbeddingModelEnum]
            The embedding model used to retrieve the chunk.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KnowledgeBaseDocumentChunkResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.chunk.get(
            documentation_id="21m00Tcm4TlvDq8ikWAM",
            chunk_id="chunk_id",
            embedding_model="e5_mistral_7b_instruct",
        )
        """
        _response = self._raw_client.get(
            documentation_id, chunk_id, embedding_model=embedding_model, request_options=request_options
        )
        return _response.data


class AsyncChunkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawChunkClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawChunkClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawChunkClient
        """
        return self._raw_client

    async def get(
        self,
        documentation_id: str,
        chunk_id: str,
        *,
        embedding_model: typing.Optional[EmbeddingModelEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> KnowledgeBaseDocumentChunkResponseModel:
        """
        Get details about a specific documentation part used by RAG.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        chunk_id : str
            The id of a document RAG chunk from the knowledge base.

        embedding_model : typing.Optional[EmbeddingModelEnum]
            The embedding model used to retrieve the chunk.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KnowledgeBaseDocumentChunkResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.chunk.get(
                documentation_id="21m00Tcm4TlvDq8ikWAM",
                chunk_id="chunk_id",
                embedding_model="e5_mistral_7b_instruct",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            documentation_id, chunk_id, embedding_model=embedding_model, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .knowledge_base_locator import KnowledgeBaseLocator
from .llm import Llm


class PromptAgentApiModelOverrideOutput(UncheckedBaseModel):
    prompt: typing.Optional[str] = pydantic.Field(default=None)
    """
    The prompt for the agent
    """

    llm: typing.Optional[Llm] = pydantic.Field(default=None)
    """
    The LLM to query with the prompt and the chat history. If using data residency, the LLM must be supported in the data residency environment
    """

    tool_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of IDs of tools used by the agent
    """

    native_mcp_server_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of Native MCP server ids to be used by the agent
    """

    knowledge_base: typing.Optional[typing.List[KnowledgeBaseLocator]] = pydantic.Field(default=None)
    """
    A list of knowledge bases to be used by the agent
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class ForcedAlignmentWordResponseModel(UncheckedBaseModel):
    """
    Model representing a single word with its timing information from the aligner.
    """

    text: str = pydantic.Field()
    """
    The word that was transcribed.
    """

    start: float = pydantic.Field()
    """
    The start time of the word in seconds.
    """

    end: float = pydantic.Field()
    """
    The end time of the word in seconds.
    """

    loss: float = pydantic.Field()
    """
    The average alignment loss/confidence score for this word, calculated from its constituent characters.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .chapter_state import ChapterState
from .chapter_statistics_response import ChapterStatisticsResponse


class ChapterResponse(UncheckedBaseModel):
    chapter_id: str = pydantic.Field()
    """
    The ID of the chapter.
    """

    name: str = pydantic.Field()
    """
    The name of the chapter.
    """

    last_conversion_date_unix: typing.Optional[int] = pydantic.Field(default=None)
    """
    The last conversion date of the chapter.
    """

    conversion_progress: typing.Optional[float] = pydantic.Field(default=None)
    """
    The conversion progress of the chapter.
    """

    can_be_downloaded: bool = pydantic.Field()
    """
    Whether the chapter can be downloaded.
    """

    state: ChapterState = pydantic.Field()
    """
    The state of the chapter.
    """

    has_video: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the chapter has a video.
    """

    has_visual_content: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the chapter has any visual content (video, image, or text clips).
    """

    voice_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of voice ids used by the chapter
    """

    statistics: typing.Optional[ChapterStatisticsResponse] = pydantic.Field(default=None)
    """
    The statistics of the chapter.
    """

    last_conversion_error: typing.Optional[str] = pydantic.Field(default=None)
    """
    The last conversion error of the chapter.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ToolType = typing.Union[
    typing.Literal[
        "system", "webhook", "client", "mcp", "workflow", "api_integration_webhook", "api_integration_mcp", "smb"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .order_id import OrderId
from .order_item_info import OrderItemInfo
from .order_state import OrderState


class OrderResponse(UncheckedBaseModel):
    order_id: OrderId = pydantic.Field()
    """
    The ID of the order.
    """

    name: str = pydantic.Field()
    """
    The display name of the order.
    """

    state: OrderState = pydantic.Field()
    """
    The current state of the order.
    """

    items: typing.List[OrderItemInfo] = pydantic.Field()
    """
    The list of items in this order with their quotes.
    """

    total_amount_usd: typing.Optional[float] = pydantic.Field(default=None)
    """
    The total price for all items in USD. Excluded from response until quotes are available.
    """

    sandbox: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this is a sandbox order that auto-progresses without producer intervention.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The timestamp when the order was created.
    """

    submitted_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when the order was submitted, if applicable.
    """

    paid_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when payment was received, if applicable.
    """

    accepted_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when the order was accepted for production, if applicable.
    """

    completed_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when the order was completed, if applicable.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class AdditionalFormatResponseModel(UncheckedBaseModel):
    requested_format: str = pydantic.Field()
    """
    The requested format.
    """

    file_extension: str = pydantic.Field()
    """
    The file extension of the additional format.
    """

    content_type: str = pydantic.Field()
    """
    The content type of the additional format.
    """

    is_base_64_encoded: typing_extensions.Annotated[bool, FieldMetadata(alias="is_base64_encoded")] = pydantic.Field()
    """
    Whether the content is base64 encoded.
    """

    content: str = pydantic.Field()
    """
    The content of the additional format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .....types.chapter_snapshot_extended_response_model import ChapterSnapshotExtendedResponseModel
from .....types.chapter_snapshots_response import ChapterSnapshotsResponse
from .raw_client import AsyncRawSnapshotsClient, RawSnapshotsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SnapshotsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSnapshotsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSnapshotsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSnapshotsClient
        """
        return self._raw_client

    def list(
        self, project_id: str, chapter_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ChapterSnapshotsResponse:
        """
        Gets information about all the snapshots of a chapter. Each snapshot can be downloaded as audio. Whenever a chapter is converted a snapshot will automatically be created.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChapterSnapshotsResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.chapters.snapshots.list(
            project_id="21m00Tcm4TlvDq8ikWAM",
            chapter_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.list(project_id, chapter_id, request_options=request_options)
        return _response.data

    def get(
        self,
        project_id: str,
        chapter_id: str,
        chapter_snapshot_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChapterSnapshotExtendedResponseModel:
        """
        Returns the chapter snapshot.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        chapter_id : str
            The ID of the chapter.

        chapter_snapshot_id : str
            The ID of the chapter snapshot.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChapterSnapshotExtendedResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.chapters.snapshots.get(
            project_id="21m00Tcm4TlvDq8ikWAM",
            chapter_id="21m00Tcm4TlvDq8ikWAM",
            chapter_snapshot_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.get(project_id, chapter_id, chapter_snapshot_id, request_options=request_options)
        return _response.data

    def stream(
        self,
        project_id: str,
        chapter_id: str,
        chapter_snapshot_id: str,
        *,
        convert_to_mpeg: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Stream the audio from a chapter snapshot. Use `GET /v1/studio/projects/{project_id}/chapters/{chapter_id}/snapshots` to return the snapshots of a chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        chapter_snapshot_id : str
            The ID of the chapter snapshot to be used. You can use the [List project chapter snapshots](/docs/api-reference/studio/get-snapshots) endpoint to list all the available snapshots.

        convert_to_mpeg : typing.Optional[bool]
            Whether to convert the audio to mpeg format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Streaming audio data

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.chapters.snapshots.stream(
            project_id="project_id",
            chapter_id="chapter_id",
            chapter_snapshot_id="chapter_snapshot_id",
        )
        """
        with self._raw_client.stream(
            project_id,
            chapter_id,
            chapter_snapshot_id,
            convert_to_mpeg=convert_to_mpeg,
            request_options=request_options,
        ) as r:
            yield from r.data


class AsyncSnapshotsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSnapshotsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSnapshotsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSnapshotsClient
        """
        return self._raw_client

    async def list(
        self, project_id: str, chapter_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ChapterSnapshotsResponse:
        """
        Gets information about all the snapshots of a chapter. Each snapshot can be downloaded as audio. Whenever a chapter is converted a snapshot will automatically be created.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChapterSnapshotsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.chapters.snapshots.list(
                project_id="21m00Tcm4TlvDq8ikWAM",
                chapter_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(project_id, chapter_id, request_options=request_options)
        return _response.data

    async def get(
        self,
        project_id: str,
        chapter_id: str,
        chapter_snapshot_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChapterSnapshotExtendedResponseModel:
        """
        Returns the chapter snapshot.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        chapter_id : str
            The ID of the chapter.

        chapter_snapshot_id : str
            The ID of the chapter snapshot.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChapterSnapshotExtendedResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.chapters.snapshots.get(
                project_id="21m00Tcm4TlvDq8ikWAM",
                chapter_id="21m00Tcm4TlvDq8ikWAM",
                chapter_snapshot_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            project_id, chapter_id, chapter_snapshot_id, request_options=request_options
        )
        return _response.data

    async def stream(
        self,
        project_id: str,
        chapter_id: str,
        chapter_snapshot_id: str,
        *,
        convert_to_mpeg: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Stream the audio from a chapter snapshot. Use `GET /v1/studio/projects/{project_id}/chapters/{chapter_id}/snapshots` to return the snapshots of a chapter.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        chapter_id : str
            The ID of the chapter to be used. You can use the [List project chapters](/docs/api-reference/studio/get-chapters) endpoint to list all the available chapters.

        chapter_snapshot_id : str
            The ID of the chapter snapshot to be used. You can use the [List project chapter snapshots](/docs/api-reference/studio/get-snapshots) endpoint to list all the available snapshots.

        convert_to_mpeg : typing.Optional[bool]
            Whether to convert the audio to mpeg format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Streaming audio data

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.chapters.snapshots.stream(
                project_id="project_id",
                chapter_id="chapter_id",
                chapter_snapshot_id="chapter_snapshot_id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.stream(
            project_id,
            chapter_id,
            chapter_snapshot_id,
            convert_to_mpeg=convert_to_mpeg,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.edit_voice_settings_response_model import EditVoiceSettingsResponseModel
from ...types.voice_settings import VoiceSettings

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSettingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_default(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[VoiceSettings]:
        """
        Gets the default settings for voices. "similarity_boost" corresponds to"Clarity + Similarity Enhancement" in the web app and "stability" corresponds to "Stability" slider in the web app.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VoiceSettings]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/voices/settings/default",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VoiceSettings,
                    construct_type(
                        type_=VoiceSettings,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, voice_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[VoiceSettings]:
        """
        Returns the settings for a specific voice. "similarity_boost" corresponds to"Clarity + Similarity Enhancement" in the web app and "stability" corresponds to "Stability" slider in the web app.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VoiceSettings]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/voices/{jsonable_encoder(voice_id)}/settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VoiceSettings,
                    construct_type(
                        type_=VoiceSettings,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, voice_id: str, *, request: VoiceSettings, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[EditVoiceSettingsResponseModel]:
        """
        Edit your settings for a specific voice. "similarity_boost" corresponds to "Clarity + Similarity Enhancement" in the web app and "stability" corresponds to "Stability" slider in the web app.

        Parameters
        ----------
        voice_id : str
            ID of the voice to be used. You can use the [Get voices](/docs/api-reference/voices/search) endpoint list all the available voices.

        request : VoiceSettings

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EditVoiceSettingsResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/voices/{jsonable_encoder(voice_id)}/settings/edit",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=VoiceSettings, direction="write"),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EditVoiceSettingsResponseModel,
                    construct_type(
                        type_=EditVoiceSettingsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSettingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[VoiceSettings]:
        """
        Gets the default settings for voices. "similarity_boost" corresponds to"Clarity + Similarity Enhancement" in the web app and "stability" corresponds to "Stability" slider in the web app.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VoiceSettings]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/voices/settings/default",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VoiceSettings,
                    construct_type(
                        type_=VoiceSettings,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, voice_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[VoiceSettings]:
        """
        Returns the settings for a specific voice. "similarity_boost" corresponds to"Clarity + Similarity Enhancement" in the web app and "stability" corresponds to "Stability" slider in the web app.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VoiceSettings]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/voices/{jsonable_encoder(voice_id)}/settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VoiceSettings,
                    construct_type(
                        type_=VoiceSettings,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, voice_id: str, *, request: VoiceSettings, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[EditVoiceSettingsResponseModel]:
        """
        Edit your settings for a specific voice. "similarity_boost" corresponds to "Clarity + Similarity Enhancement" in the web app and "stability" corresponds to "Stability" slider in the web app.

        Parameters
        ----------
        voice_id : str
            ID of the voice to be used. You can use the [Get voices](/docs/api-reference/voices/search) endpoint list all the available voices.

        request : VoiceSettings

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EditVoiceSettingsResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/voices/{jsonable_encoder(voice_id)}/settings/edit",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=VoiceSettings, direction="write"),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EditVoiceSettingsResponseModel,
                    construct_type(
                        type_=EditVoiceSettingsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.conv_ai_secret_locator import ConvAiSecretLocator
from ...types.mcp_approval_policy import McpApprovalPolicy
from ...types.mcp_server_config_input import McpServerConfigInput
from ...types.mcp_server_response_model import McpServerResponseModel
from ...types.mcp_servers_response_model import McpServersResponseModel
from ...types.pre_tool_speech_mode import PreToolSpeechMode
from ...types.tool_call_sound_behavior import ToolCallSoundBehavior
from ...types.tool_call_sound_type import ToolCallSoundType
from ...types.tool_execution_mode import ToolExecutionMode
from .raw_client import AsyncRawMcpServersClient, RawMcpServersClient
from .types.mcp_server_config_update_request_model_auth_connection import (
    McpServerConfigUpdateRequestModelAuthConnection,
)
from .types.mcp_server_config_update_request_model_request_headers_value import (
    McpServerConfigUpdateRequestModelRequestHeadersValue,
)

if typing.TYPE_CHECKING:
    from .approval_policy.client import ApprovalPolicyClient, AsyncApprovalPolicyClient
    from .tool_approvals.client import AsyncToolApprovalsClient, ToolApprovalsClient
    from .tool_configs.client import AsyncToolConfigsClient, ToolConfigsClient
    from .tools.client import AsyncToolsClient, ToolsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class McpServersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMcpServersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._tools: typing.Optional[ToolsClient] = None
        self._approval_policy: typing.Optional[ApprovalPolicyClient] = None
        self._tool_approvals: typing.Optional[ToolApprovalsClient] = None
        self._tool_configs: typing.Optional[ToolConfigsClient] = None

    @property
    def with_raw_response(self) -> RawMcpServersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMcpServersClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> McpServersResponseModel:
        """
        Retrieve all MCP server configurations available in the workspace.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServersResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.mcp_servers.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def create(
        self, *, config: McpServerConfigInput, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerResponseModel:
        """
        Create a new MCP server configuration in the workspace.

        Parameters
        ----------
        config : McpServerConfigInput
            Configuration details for the MCP Server.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs, McpServerConfigInput

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.mcp_servers.create(
            config=McpServerConfigInput(
                url="url",
                name="name",
            ),
        )
        """
        _response = self._raw_client.create(config=config, request_options=request_options)
        return _response.data

    def get(
        self, mcp_server_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerResponseModel:
        """
        Retrieve a specific MCP server configuration from the workspace.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.mcp_servers.get(
            mcp_server_id="mcp_server_id",
        )
        """
        _response = self._raw_client.get(mcp_server_id, request_options=request_options)
        return _response.data

    def delete(self, mcp_server_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> typing.Any:
        """
        Delete a specific MCP server configuration from the workspace.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.mcp_servers.delete(
            mcp_server_id="mcp_server_id",
        )
        """
        _response = self._raw_client.delete(mcp_server_id, request_options=request_options)
        return _response.data

    def update(
        self,
        mcp_server_id: str,
        *,
        approval_policy: typing.Optional[McpApprovalPolicy] = OMIT,
        force_pre_tool_speech: typing.Optional[bool] = OMIT,
        pre_tool_speech: typing.Optional[PreToolSpeechMode] = OMIT,
        disable_interruptions: typing.Optional[bool] = OMIT,
        tool_call_sound: typing.Optional[ToolCallSoundType] = OMIT,
        tool_call_sound_behavior: typing.Optional[ToolCallSoundBehavior] = OMIT,
        execution_mode: typing.Optional[ToolExecutionMode] = OMIT,
        response_timeout_secs: typing.Optional[int] = OMIT,
        request_headers: typing.Optional[
            typing.Dict[str, typing.Optional[McpServerConfigUpdateRequestModelRequestHeadersValue]]
        ] = OMIT,
        disable_compression: typing.Optional[bool] = OMIT,
        secret_token: typing.Optional[ConvAiSecretLocator] = OMIT,
        auth_connection: typing.Optional[McpServerConfigUpdateRequestModelAuthConnection] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> McpServerResponseModel:
        """
        Update the configuration settings for an MCP server.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        approval_policy : typing.Optional[McpApprovalPolicy]
            The approval mode to set for the MCP server

        force_pre_tool_speech : typing.Optional[bool]
            DEPRECATED: use `pre_tool_speech` instead. If set, overrides the server's force_pre_tool_speech setting for this tool.

        pre_tool_speech : typing.Optional[PreToolSpeechMode]
            If set, overrides the server's pre_tool_speech setting for this tool.

        disable_interruptions : typing.Optional[bool]
            If set, overrides the server's disable_interruptions setting for this tool

        tool_call_sound : typing.Optional[ToolCallSoundType]
            Predefined tool call sound type to play during tool execution for all tools from this MCP server

        tool_call_sound_behavior : typing.Optional[ToolCallSoundBehavior]
            Determines when the tool call sound should play for all tools from this MCP server

        execution_mode : typing.Optional[ToolExecutionMode]
            If set, overrides the server's execution_mode setting for this tool

        response_timeout_secs : typing.Optional[int]
            The maximum time in seconds to wait for each MCP tool call to complete.

        request_headers : typing.Optional[typing.Dict[str, typing.Optional[McpServerConfigUpdateRequestModelRequestHeadersValue]]]
            The headers to include in requests to the MCP server

        disable_compression : typing.Optional[bool]
            Whether to disable HTTP compression for this MCP server

        secret_token : typing.Optional[ConvAiSecretLocator]
            Optional secret token for authentication with this MCP server

        auth_connection : typing.Optional[McpServerConfigUpdateRequestModelAuthConnection]
            Optional auth connection to use for authentication with this MCP server

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.mcp_servers.update(
            mcp_server_id="mcp_server_id",
        )
        """
        _response = self._raw_client.update(
            mcp_server_id,
            approval_policy=approval_policy,
            force_pre_tool_speech=force_pre_tool_speech,
            pre_tool_speech=pre_tool_speech,
            disable_interruptions=disable_interruptions,
            tool_call_sound=tool_call_sound,
            tool_call_sound_behavior=tool_call_sound_behavior,
            execution_mode=execution_mode,
            response_timeout_secs=response_timeout_secs,
            request_headers=request_headers,
            disable_compression=disable_compression,
            secret_token=secret_token,
            auth_connection=auth_connection,
            request_options=request_options,
        )
        return _response.data

    @property
    def tools(self):
        if self._tools is None:
            from .tools.client import ToolsClient  # noqa: E402

            self._tools = ToolsClient(client_wrapper=self._client_wrapper)
        return self._tools

    @property
    def approval_policy(self):
        if self._approval_policy is None:
            from .approval_policy.client import ApprovalPolicyClient  # noqa: E402

            self._approval_policy = ApprovalPolicyClient(client_wrapper=self._client_wrapper)
        return self._approval_policy

    @property
    def tool_approvals(self):
        if self._tool_approvals is None:
            from .tool_approvals.client import ToolApprovalsClient  # noqa: E402

            self._tool_approvals = ToolApprovalsClient(client_wrapper=self._client_wrapper)
        return self._tool_approvals

    @property
    def tool_configs(self):
        if self._tool_configs is None:
            from .tool_configs.client import ToolConfigsClient  # noqa: E402

            self._tool_configs = ToolConfigsClient(client_wrapper=self._client_wrapper)
        return self._tool_configs


class AsyncMcpServersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMcpServersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._tools: typing.Optional[AsyncToolsClient] = None
        self._approval_policy: typing.Optional[AsyncApprovalPolicyClient] = None
        self._tool_approvals: typing.Optional[AsyncToolApprovalsClient] = None
        self._tool_configs: typing.Optional[AsyncToolConfigsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawMcpServersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMcpServersClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> McpServersResponseModel:
        """
        Retrieve all MCP server configurations available in the workspace.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServersResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.mcp_servers.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def create(
        self, *, config: McpServerConfigInput, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerResponseModel:
        """
        Create a new MCP server configuration in the workspace.

        Parameters
        ----------
        config : McpServerConfigInput
            Configuration details for the MCP Server.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, McpServerConfigInput

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.mcp_servers.create(
                config=McpServerConfigInput(
                    url="url",
                    name="name",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(config=config, request_options=request_options)
        return _response.data

    async def get(
        self, mcp_server_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerResponseModel:
        """
        Retrieve a specific MCP server configuration from the workspace.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.mcp_servers.get(
                mcp_server_id="mcp_server_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(mcp_server_id, request_options=request_options)
        return _response.data

    async def delete(
        self, mcp_server_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Any:
        """
        Delete a specific MCP server configuration from the workspace.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.mcp_servers.delete(
                mcp_server_id="mcp_server_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(mcp_server_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        mcp_server_id: str,
        *,
        approval_policy: typing.Optional[McpApprovalPolicy] = OMIT,
        force_pre_tool_speech: typing.Optional[bool] = OMIT,
        pre_tool_speech: typing.Optional[PreToolSpeechMode] = OMIT,
        disable_interruptions: typing.Optional[bool] = OMIT,
        tool_call_sound: typing.Optional[ToolCallSoundType] = OMIT,
        tool_call_sound_behavior: typing.Optional[ToolCallSoundBehavior] = OMIT,
        execution_mode: typing.Optional[ToolExecutionMode] = OMIT,
        response_timeout_secs: typing.Optional[int] = OMIT,
        request_headers: typing.Optional[
            typing.Dict[str, typing.Optional[McpServerConfigUpdateRequestModelRequestHeadersValue]]
        ] = OMIT,
        disable_compression: typing.Optional[bool] = OMIT,
        secret_token: typing.Optional[ConvAiSecretLocator] = OMIT,
        auth_connection: typing.Optional[McpServerConfigUpdateRequestModelAuthConnection] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> McpServerResponseModel:
        """
        Update the configuration settings for an MCP server.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        approval_policy : typing.Optional[McpApprovalPolicy]
            The approval mode to set for the MCP server

        force_pre_tool_speech : typing.Optional[bool]
            DEPRECATED: use `pre_tool_speech` instead. If set, overrides the server's force_pre_tool_speech setting for this tool.

        pre_tool_speech : typing.Optional[PreToolSpeechMode]
            If set, overrides the server's pre_tool_speech setting for this tool.

        disable_interruptions : typing.Optional[bool]
            If set, overrides the server's disable_interruptions setting for this tool

        tool_call_sound : typing.Optional[ToolCallSoundType]
            Predefined tool call sound type to play during tool execution for all tools from this MCP server

        tool_call_sound_behavior : typing.Optional[ToolCallSoundBehavior]
            Determines when the tool call sound should play for all tools from this MCP server

        execution_mode : typing.Optional[ToolExecutionMode]
            If set, overrides the server's execution_mode setting for this tool

        response_timeout_secs : typing.Optional[int]
            The maximum time in seconds to wait for each MCP tool call to complete.

        request_headers : typing.Optional[typing.Dict[str, typing.Optional[McpServerConfigUpdateRequestModelRequestHeadersValue]]]
            The headers to include in requests to the MCP server

        disable_compression : typing.Optional[bool]
            Whether to disable HTTP compression for this MCP server

        secret_token : typing.Optional[ConvAiSecretLocator]
            Optional secret token for authentication with this MCP server

        auth_connection : typing.Optional[McpServerConfigUpdateRequestModelAuthConnection]
            Optional auth connection to use for authentication with this MCP server

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.mcp_servers.update(
                mcp_server_id="mcp_server_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            mcp_server_id,
            approval_policy=approval_policy,
            force_pre_tool_speech=force_pre_tool_speech,
            pre_tool_speech=pre_tool_speech,
            disable_interruptions=disable_interruptions,
            tool_call_sound=tool_call_sound,
            tool_call_sound_behavior=tool_call_sound_behavior,
            execution_mode=execution_mode,
            response_timeout_secs=response_timeout_secs,
            request_headers=request_headers,
            disable_compression=disable_compression,
            secret_token=secret_token,
            auth_connection=auth_connection,
            request_options=request_options,
        )
        return _response.data

    @property
    def tools(self):
        if self._tools is None:
            from .tools.client import AsyncToolsClient  # noqa: E402

            self._tools = AsyncToolsClient(client_wrapper=self._client_wrapper)
        return self._tools

    @property
    def approval_policy(self):
        if self._approval_policy is None:
            from .approval_policy.client import AsyncApprovalPolicyClient  # noqa: E402

            self._approval_policy = AsyncApprovalPolicyClient(client_wrapper=self._client_wrapper)
        return self._approval_policy

    @property
    def tool_approvals(self):
        if self._tool_approvals is None:
            from .tool_approvals.client import AsyncToolApprovalsClient  # noqa: E402

            self._tool_approvals = AsyncToolApprovalsClient(client_wrapper=self._client_wrapper)
        return self._tool_approvals

    @property
    def tool_configs(self):
        if self._tool_configs is None:
            from .tool_configs.client import AsyncToolConfigsClient  # noqa: E402

            self._tool_configs = AsyncToolConfigsClient(client_wrapper=self._client_wrapper)
        return self._tool_configs



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .mcp_approval_policy import McpApprovalPolicy
from .mcp_server_config_output_auth_connection import McpServerConfigOutputAuthConnection
from .mcp_server_config_output_request_headers_value import McpServerConfigOutputRequestHeadersValue
from .mcp_server_config_output_secret_token import McpServerConfigOutputSecretToken
from .mcp_server_config_output_url import McpServerConfigOutputUrl
from .mcp_server_transport import McpServerTransport
from .mcp_tool_approval_hash import McpToolApprovalHash
from .mcp_tool_config_override_output import McpToolConfigOverrideOutput
from .pre_tool_speech_mode import PreToolSpeechMode
from .tool_call_sound_behavior import ToolCallSoundBehavior
from .tool_call_sound_type import ToolCallSoundType
from .tool_execution_mode import ToolExecutionMode


class McpServerConfigOutput(UncheckedBaseModel):
    approval_policy: typing.Optional[McpApprovalPolicy] = None
    tool_approval_hashes: typing.Optional[typing.List[McpToolApprovalHash]] = pydantic.Field(default=None)
    """
    List of tool approval hashes for per-tool approval when approval_policy is REQUIRE_APPROVAL_PER_TOOL
    """

    transport: typing.Optional[McpServerTransport] = pydantic.Field(default=None)
    """
    The transport type used to connect to the MCP server
    """

    url: McpServerConfigOutputUrl = pydantic.Field()
    """
    The URL of the MCP server, if this contains a secret please store as a workspace secret, otherwise store as a plain string. Must use https
    """

    secret_token: typing.Optional[McpServerConfigOutputSecretToken] = pydantic.Field(default=None)
    """
    The secret token (Authorization header) stored as a workspace secret or in-place secret
    """

    request_headers: typing.Optional[typing.Dict[str, McpServerConfigOutputRequestHeadersValue]] = pydantic.Field(
        default=None
    )
    """
    The headers included in the request
    """

    auth_connection: typing.Optional[McpServerConfigOutputAuthConnection] = pydantic.Field(default=None)
    """
    Optional auth connection to use for authentication with this MCP server
    """

    name: str
    description: typing.Optional[str] = None
    force_pre_tool_speech: typing.Optional[bool] = pydantic.Field(default=None)
    """
    DEPRECATED: use `pre_tool_speech` instead. If true, all tools from this MCP server will require pre-tool execution speech.
    """

    pre_tool_speech: typing.Optional[PreToolSpeechMode] = pydantic.Field(default=None)
    """
    Controls whether the agent speaks before this tool is called. 'auto' (default) decides based on recent tool latency, 'force' always asks the agent to speak, 'off' fully opts out regardless of latency. Applies to every tool from this MCP server unless overridden per tool.
    """

    disable_interruptions: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, the user will not be able to interrupt the agent while any tool from this MCP server is running.
    """

    tool_call_sound: typing.Optional[ToolCallSoundType] = pydantic.Field(default=None)
    """
    Predefined tool call sound type to play during tool execution for all tools from this MCP server
    """

    tool_call_sound_behavior: typing.Optional[ToolCallSoundBehavior] = pydantic.Field(default=None)
    """
    Determines when the tool call sound should play for all tools from this MCP server
    """

    execution_mode: typing.Optional[ToolExecutionMode] = pydantic.Field(default=None)
    """
    Determines when and how all tools from this MCP server execute: 'immediate' executes the tool right away when requested by the LLM, 'post_tool_speech' waits for the agent to finish speaking before executing, 'async' runs the tool in the background without blocking - best for long-running operations.
    """

    response_timeout_secs: typing.Optional[int] = pydantic.Field(default=None)
    """
    The maximum time in seconds to wait for each MCP tool call to complete. Must be between 5 and 300 seconds (inclusive).
    """

    tool_config_overrides: typing.Optional[typing.List[McpToolConfigOverrideOutput]] = pydantic.Field(default=None)
    """
    List of per-tool configuration overrides that override the server-level defaults for specific tools
    """

    disable_compression: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to disable HTTP compression for this MCP server. Enable this if the server does not support compressed responses.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .discount_response_model import DiscountResponseModel
from .invoice_response_model_payment_intent_status import InvoiceResponseModelPaymentIntentStatus
from .invoice_response_model_payment_intent_statusses_item import InvoiceResponseModelPaymentIntentStatussesItem


class InvoiceResponse(UncheckedBaseModel):
    amount_due_cents: int = pydantic.Field()
    """
    The amount due in cents.
    """

    subtotal_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The subtotal amount in cents before tax (exclusive of tax and discounts).
    """

    tax_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The tax amount in cents.
    """

    discount_percent_off: typing.Optional[float] = pydantic.Field(default=None)
    """
    Deprecated. Use [discounts] instead. The discount applied to the invoice. E.g. [20.0f] for 20% off.
    """

    discount_amount_off: typing.Optional[float] = pydantic.Field(default=None)
    """
    Deprecated. Use [discounts] instead. The discount applied to the invoice. E.g. [20.0f] for 20 cents off.
    """

    discounts: typing.List[DiscountResponseModel] = pydantic.Field()
    """
    The discounts applied to the invoice.
    """

    next_payment_attempt_unix: int = pydantic.Field()
    """
    The Unix timestamp of the next payment attempt. -1 when there is no next payment attempt.
    """

    payment_intent_status: typing.Optional[InvoiceResponseModelPaymentIntentStatus] = pydantic.Field(default=None)
    """
    Deprecated. Use [payment_intent_statusses] instead. The status of this invoice's first payment intent. None when there is no payment intent.
    """

    payment_intent_statusses: typing.List[InvoiceResponseModelPaymentIntentStatussesItem] = pydantic.Field()
    """
    The statuses of this invoice's payment intents. Empty list when there are no payment intents.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""Tests for SpeechEngineSession — mirrors SpeechEngineSession.test.ts."""

import asyncio
import json
import typing

import pytest

from elevenlabs.speech_engine import (
    CLOSE,
    DISCONNECTED,
    ERROR,
    INIT,
    USER_TRANSCRIPT,
    ConversationMessage,
    SpeechEngineSession,
)


# ---------------------------------------------------------------------------
# MockWebSocket
# ---------------------------------------------------------------------------


class MockWebSocket:
    """In-memory WebSocket stand-in backed by an asyncio.Queue."""

    def __init__(self) -> None:
        self._inbox = asyncio.Queue()  # type: asyncio.Queue[typing.Any]
        self.sent = []  # type: typing.List[str]
        self.closed = False

    async def recv(self) -> str:
        msg = await self._inbox.get()
        if msg is _CLOSE_SENTINEL:
            raise ConnectionError("connection closed")
        return msg

    async def send(self, data: str) -> None:
        self.sent.append(data)

    async def close(self) -> None:
        self.closed = True

    # -- test helpers --

    def receive_message(self, msg: typing.Dict[str, typing.Any]) -> None:
        """Inject an incoming message from the "ElevenLabs API"."""
        self._inbox.put_nowait(json.dumps(msg))

    def receive_raw(self, data: str) -> None:
        """Inject raw (possibly invalid) data."""
        self._inbox.put_nowait(data)

    def simulate_disconnect(self) -> None:
        """Simulate a WebSocket disconnection."""
        self._inbox.put_nowait(_CLOSE_SENTINEL)


_CLOSE_SENTINEL = object()


# ---------------------------------------------------------------------------
# Fixtures
# ---------------------------------------------------------------------------

TRANSCRIPT = [
    {"role": "agent", "content": "how can I help you today?"},
    {"role": "user", "content": "I need a pizza"},
]

TRANSCRIPT_2 = [
    {"role": "agent", "content": "how can I help you today?"},
    {"role": "user", "content": "I need a pizza"},
    {"role": "agent", "content": "what size?"},
    {"role": "user", "content": "large"},
]


@pytest.fixture
def ws() -> MockWebSocket:
    return MockWebSocket()


@pytest.fixture
def session(ws: MockWebSocket) -> SpeechEngineSession:
    return SpeechEngineSession(ws)


# ---------------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------------


def _parsed_sent(ws: MockWebSocket) -> typing.List[typing.Dict[str, typing.Any]]:
    return [json.loads(s) for s in ws.sent]


async def _run_until_idle(session: SpeechEngineSession, ws: MockWebSocket) -> None:
    """Drive the session run loop, then disconnect so it returns."""
    ws.simulate_disconnect()
    await session.run()


# ---------------------------------------------------------------------------
# init event
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_emits_init_with_conversation_id(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    captured = []  # type: typing.List[str]

    async def handler(conversation_id: str) -> None:
        captured.append(conversation_id)

    session.on(INIT, handler)
    ws.receive_message({"type": "init", "conversation_id": "conv_42"})
    await _run_until_idle(session, ws)

    assert captured == ["conv_42"]
    assert session.conversation_id == "conv_42"


# ---------------------------------------------------------------------------
# user_transcript events
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_emits_user_transcript_with_conversation_history(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    captured = []  # type: typing.List[typing.List[ConversationMessage]]

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        captured.append(transcript)

    session.on(USER_TRANSCRIPT, handler)
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 1}
    )
    await _run_until_idle(session, ws)

    assert len(captured) == 1
    assert len(captured[0]) == 2
    assert captured[0][0].role == "agent"
    assert captured[0][0].content == "how can I help you today?"
    assert captured[0][1].role == "user"
    assert captured[0][1].content == "I need a pizza"


@pytest.mark.asyncio
async def test_cancels_previous_handler_on_new_transcript(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    """When a new transcript arrives the previous handler task is cancelled."""
    cancelled = asyncio.Event()

    async def slow_handler(transcript: typing.List[ConversationMessage]) -> None:
        try:
            await asyncio.sleep(10)
        except asyncio.CancelledError:
            cancelled.set()
            raise

    session.on(USER_TRANSCRIPT, slow_handler)
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 1}
    )
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT_2, "event_id": 2}
    )
    await _run_until_idle(session, ws)

    assert cancelled.is_set()


# ---------------------------------------------------------------------------
# ping / pong
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_auto_responds_to_ping(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message({"type": "ping"})
    await _run_until_idle(session, ws)

    assert len(ws.sent) == 1
    assert json.loads(ws.sent[0]) == {"type": "pong"}


# ---------------------------------------------------------------------------
# close event (protocol-level)
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_emits_close_and_cancels_handler_on_close_message(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    close_called = asyncio.Event()
    cancelled = asyncio.Event()

    async def close_handler() -> None:
        close_called.set()

    async def slow_handler(transcript: typing.List[ConversationMessage]) -> None:
        try:
            await asyncio.sleep(10)
        except asyncio.CancelledError:
            cancelled.set()
            raise

    session.on(CLOSE, close_handler)
    session.on(USER_TRANSCRIPT, slow_handler)

    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 1}
    )
    ws.receive_message({"type": "close"})
    await _run_until_idle(session, ws)

    assert close_called.is_set()
    assert cancelled.is_set()


# ---------------------------------------------------------------------------
# error event (protocol-level)
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_emits_error_on_protocol_error_message(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    errors = []  # type: typing.List[Exception]

    async def handler(err: Exception) -> None:
        errors.append(err)

    session.on(ERROR, handler)
    ws.receive_message({"type": "error", "message": "something went wrong"})
    await _run_until_idle(session, ws)

    assert len(errors) == 1
    assert isinstance(errors[0], Exception)
    assert str(errors[0]) == "something went wrong"


# ---------------------------------------------------------------------------
# WebSocket-level events
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_emits_disconnected_when_websocket_closes(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    disconnected = asyncio.Event()

    async def handler() -> None:
        disconnected.set()

    session.on(DISCONNECTED, handler)
    ws.simulate_disconnect()
    await session.run()

    assert disconnected.is_set()
    assert not session.is_open


@pytest.mark.asyncio
async def test_cancels_handler_when_websocket_closes(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    cancelled = asyncio.Event()

    async def slow_handler(transcript: typing.List[ConversationMessage]) -> None:
        try:
            await asyncio.sleep(10)
        except asyncio.CancelledError:
            cancelled.set()
            raise

    session.on(USER_TRANSCRIPT, slow_handler)
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 1}
    )
    # Give the handler task a moment to start before disconnecting.
    await asyncio.sleep(0)
    ws.simulate_disconnect()
    await session.run()

    # The disconnect triggers _cancel_current in the finally block.
    assert cancelled.is_set()


@pytest.mark.asyncio
async def test_emits_error_on_malformed_json(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    errors = []  # type: typing.List[Exception]

    async def handler(err: Exception) -> None:
        errors.append(err)

    session.on(ERROR, handler)
    ws.receive_raw("not json")
    await _run_until_idle(session, ws)

    assert len(errors) == 1


@pytest.mark.asyncio
async def test_ignores_unknown_message_types(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    errors = []  # type: typing.List[Exception]

    async def handler(err: Exception) -> None:
        errors.append(err)

    session.on(ERROR, handler)
    ws.receive_message({"type": "unknown_future_event", "data": 123})
    await _run_until_idle(session, ws)

    assert len(errors) == 0


# ---------------------------------------------------------------------------
# sendResponse (string)
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_sends_string_response_with_event_id_and_is_final(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 5}
    )

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        await session.send_response("The answer is 42")

    session.on(USER_TRANSCRIPT, handler)
    await _run_until_idle(session, ws)

    sent = _parsed_sent(ws)
    assert len(sent) == 2
    assert sent[0] == {
        "type": "agent_response",
        "content": "The answer is 42",
        "event_id": 5,
        "is_final": False,
    }
    assert sent[1] == {
        "type": "agent_response",
        "content": "",
        "event_id": 5,
        "is_final": True,
    }


@pytest.mark.asyncio
async def test_raises_when_sending_after_close(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    session.close()
    with pytest.raises(RuntimeError, match="session is closed"):
        await session.send_response("too late")


# ---------------------------------------------------------------------------
# sendResponse (streaming)
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_streams_chunks_with_is_final_false_then_terminator(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 3}
    )

    async def tokens():  # type: ignore[return]
        yield "Hello"
        yield " world"

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        await session.send_response(tokens())

    session.on(USER_TRANSCRIPT, handler)
    await _run_until_idle(session, ws)

    sent = _parsed_sent(ws)
    assert len(sent) == 3
    assert sent[0] == {
        "type": "agent_response",
        "content": "Hello",
        "event_id": 3,
        "is_final": False,
    }
    assert sent[1] == {
        "type": "agent_response",
        "content": " world",
        "event_id": 3,
        "is_final": False,
    }
    assert sent[2] == {
        "type": "agent_response",
        "content": "",
        "event_id": 3,
        "is_final": True,
    }


@pytest.mark.asyncio
async def test_stops_streaming_when_session_closed_mid_stream(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 7}
    )

    async def slow_tokens():  # type: ignore[return]
        yield "first"
        yield "second"
        await asyncio.sleep(0.1)
        yield "third"

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        await session.send_response(slow_tokens())

    session.on(USER_TRANSCRIPT, handler)

    # Start the run loop in the background.
    task = asyncio.create_task(session.run())
    # Let the handler task progress through the first two yields.
    await asyncio.sleep(0.05)
    session.close()
    await asyncio.sleep(0.15)
    task.cancel()
    try:
        await task
    except asyncio.CancelledError:
        pass

    sent = _parsed_sent(ws)
    chunks = [m for m in sent if m.get("type") == "agent_response"]
    # "first" and "second" are sent; session closes before "third".
    assert len(chunks) == 2
    assert chunks[0] == {
        "type": "agent_response",
        "content": "first",
        "event_id": 7,
        "is_final": False,
    }
    assert chunks[1] == {
        "type": "agent_response",
        "content": "second",
        "event_id": 7,
        "is_final": False,
    }


# ---------------------------------------------------------------------------
# sendResponse (LLM stream formats)
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_extracts_text_from_openai_responses_api(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 10}
    )

    async def openai_responses_stream():  # type: ignore[return]
        yield {"type": "response.created", "response": {}}
        yield {"type": "response.output_text.delta", "delta": "Hello"}
        yield {"type": "response.output_text.delta", "delta": " world"}
        yield {"type": "response.output_text.done", "text": "Hello world"}
        yield {"type": "response.completed", "response": {}}

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        await session.send_response(openai_responses_stream())

    session.on(USER_TRANSCRIPT, handler)
    await _run_until_idle(session, ws)

    sent = _parsed_sent(ws)
    assert len(sent) == 3
    assert sent[0] == {
        "type": "agent_response",
        "content": "Hello",
        "event_id": 10,
        "is_final": False,
    }
    assert sent[1] == {
        "type": "agent_response",
        "content": " world",
        "event_id": 10,
        "is_final": False,
    }
    assert sent[2] == {
        "type": "agent_response",
        "content": "",
        "event_id": 10,
        "is_final": True,
    }


@pytest.mark.asyncio
async def test_extracts_text_from_openai_chat_completions_api(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 11}
    )

    async def openai_chat_stream():  # type: ignore[return]
        yield {"choices": [{"delta": {"role": "assistant"}}]}
        yield {"choices": [{"delta": {"content": "Hi"}}]}
        yield {"choices": [{"delta": {"content": " there"}}]}
        yield {"choices": [{"delta": {}}]}

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        await session.send_response(openai_chat_stream())

    session.on(USER_TRANSCRIPT, handler)
    await _run_until_idle(session, ws)

    sent = _parsed_sent(ws)
    assert len(sent) == 3
    assert sent[0] == {
        "type": "agent_response",
        "content": "Hi",
        "event_id": 11,
        "is_final": False,
    }
    assert sent[1] == {
        "type": "agent_response",
        "content": " there",
        "event_id": 11,
        "is_final": False,
    }
    assert sent[2] == {
        "type": "agent_response",
        "content": "",
        "event_id": 11,
        "is_final": True,
    }


@pytest.mark.asyncio
async def test_extracts_text_from_anthropic_messages_api(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 12}
    )

    async def anthropic_stream():  # type: ignore[return]
        yield {"type": "message_start", "message": {}}
        yield {"type": "content_block_start", "content_block": {"type": "text", "text": ""}}
        yield {"type": "content_block_delta", "delta": {"type": "text_delta", "text": "Good"}}
        yield {"type": "content_block_delta", "delta": {"type": "text_delta", "text": " morning"}}
        yield {"type": "content_block_stop"}
        yield {"type": "message_stop"}

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        await session.send_response(anthropic_stream())

    session.on(USER_TRANSCRIPT, handler)
    await _run_until_idle(session, ws)

    sent = _parsed_sent(ws)
    assert len(sent) == 3
    assert sent[0] == {
        "type": "agent_response",
        "content": "Good",
        "event_id": 12,
        "is_final": False,
    }
    assert sent[1] == {
        "type": "agent_response",
        "content": " morning",
        "event_id": 12,
        "is_final": False,
    }
    assert sent[2] == {
        "type": "agent_response",
        "content": "",
        "event_id": 12,
        "is_final": True,
    }


@pytest.mark.asyncio
async def test_extracts_text_from_gemini_api(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 13}
    )

    async def gemini_stream():  # type: ignore[return]
        yield {"candidates": [{"content": {"parts": [{"text": "Hey"}], "role": "model"}}]}
        yield {"candidates": [{"content": {"parts": [{"text": " buddy"}], "role": "model"}}]}

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        await session.send_response(gemini_stream())

    session.on(USER_TRANSCRIPT, handler)
    await _run_until_idle(session, ws)

    sent = _parsed_sent(ws)
    assert len(sent) == 3
    assert sent[0] == {
        "type": "agent_response",
        "content": "Hey",
        "event_id": 13,
        "is_final": False,
    }
    assert sent[1] == {
        "type": "agent_response",
        "content": " buddy",
        "event_id": 13,
        "is_final": False,
    }
    assert sent[2] == {
        "type": "agent_response",
        "content": "",
        "event_id": 13,
        "is_final": True,
    }


@pytest.mark.asyncio
async def test_skips_unrecognized_stream_events(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 14}
    )

    async def mixed_stream():  # type: ignore[return]
        yield {"type": "unknown_event", "data": 123}
        yield {"type": "response.output_text.delta", "delta": "text"}
        yield 42
        yield None

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        await session.send_response(mixed_stream())

    session.on(USER_TRANSCRIPT, handler)
    await _run_until_idle(session, ws)

    sent = _parsed_sent(ws)
    assert len(sent) == 2
    assert sent[0] == {
        "type": "agent_response",
        "content": "text",
        "event_id": 14,
        "is_final": False,
    }
    assert sent[1] == {
        "type": "agent_response",
        "content": "",
        "event_id": 14,
        "is_final": True,
    }


# ---------------------------------------------------------------------------
# event_id tracking across interrupts
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_stamps_correct_event_id_after_interrupt(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 1}
    )
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT_2, "event_id": 2}
    )

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        # Non-blocking so both handlers complete within the run loop.
        await session.send_response("response")

    session.on(USER_TRANSCRIPT, handler)
    await _run_until_idle(session, ws)

    sent = _parsed_sent(ws)
    # Each sendResponse emits content + terminator = 2 messages.
    # The first handler's task may or may not complete before cancellation;
    # the second handler's response should use event_id 2.
    # Filter to the final handler's messages (event_id=2):
    id2 = [m for m in sent if m.get("event_id") == 2]
    assert len(id2) == 2
    assert id2[0]["content"] == "response"
    assert id2[0]["is_final"] is False
    assert id2[1]["content"] == ""
    assert id2[1]["is_final"] is True


# ---------------------------------------------------------------------------
# close()
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_close_is_idempotent(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    session.close()
    session.close()
    assert not session.is_open


# ---------------------------------------------------------------------------
# Event constants
# ---------------------------------------------------------------------------


def test_event_constants() -> None:
    assert INIT == "init"
    assert USER_TRANSCRIPT == "user_transcript"
    assert CLOSE == "close"
    assert ERROR == "error"
    assert DISCONNECTED == "disconnected"


# ---------------------------------------------------------------------------
# on / off / once
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_off_removes_handler(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    calls = []  # type: typing.List[str]

    async def handler(conversation_id: str) -> None:
        calls.append(conversation_id)

    session.on(INIT, handler)
    session.off(INIT, handler)

    ws.receive_message({"type": "init", "conversation_id": "conv_1"})
    await _run_until_idle(session, ws)

    assert calls == []


@pytest.mark.asyncio
async def test_once_fires_only_once(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    calls = []  # type: typing.List[str]

    async def handler(conversation_id: str) -> None:
        calls.append(conversation_id)

    session.once(INIT, handler)

    ws.receive_message({"type": "init", "conversation_id": "conv_1"})
    ws.receive_message({"type": "init", "conversation_id": "conv_2"})
    await _run_until_idle(session, ws)

    assert calls == ["conv_1"]


# ---------------------------------------------------------------------------
# Transcript deduplication
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_skips_duplicate_event_id(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    """Duplicate transcripts with the same event_id are ignored."""
    call_count = 0

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        nonlocal call_count
        call_count += 1
        await asyncio.sleep(0.5)

    session.on(USER_TRANSCRIPT, handler)
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 30}
    )
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 30}
    )
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 30}
    )
    await _run_until_idle(session, ws)

    assert call_count == 1


@pytest.mark.asyncio
async def test_does_not_skip_different_event_id(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    """Different event_ids are processed normally."""
    call_count = 0

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        nonlocal call_count
        call_count += 1

    session.on(USER_TRANSCRIPT, handler)
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT, "event_id": 1}
    )
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT_2, "event_id": 2}
    )
    await _run_until_idle(session, ws)

    assert call_count == 2


@pytest.mark.asyncio
async def test_does_not_deduplicate_when_event_id_is_none(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    """Transcripts without event_id should never be deduplicated."""
    call_count = 0

    async def handler(transcript: typing.List[ConversationMessage]) -> None:
        nonlocal call_count
        call_count += 1

    session.on(USER_TRANSCRIPT, handler)
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT}
    )
    ws.receive_message(
        {"type": "user_transcript", "user_transcript": TRANSCRIPT_2}
    )
    await _run_until_idle(session, ws)

    assert call_count == 2


# ---------------------------------------------------------------------------
# send_response outside on_transcript
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_send_response_warns_outside_transcript(
    ws: MockWebSocket, session: SpeechEngineSession
) -> None:
    """send_response before any transcript should warn and not send."""
    await session.send_response("hello")
    assert len(ws.sent) == 0



# This file was auto-generated by Fern from our API Definition.

import typing

WorkspaceResourceType = typing.Union[
    typing.Literal[
        "voice",
        "voice_collection",
        "pronunciation_dictionary",
        "dubbing",
        "project",
        "convai_agents",
        "convai_knowledge_base_documents",
        "convai_tools",
        "convai_settings",
        "convai_secrets",
        "workspace_auth_connections",
        "convai_phone_numbers",
        "convai_mcp_servers",
        "convai_api_integration_connections",
        "convai_api_integration_trigger_connections",
        "convai_batch_calls",
        "convai_agent_response_tests",
        "convai_test_suite_invocations",
        "convai_crawl_jobs",
        "convai_crawl_tasks",
        "convai_whatsapp_accounts",
        "convai_agent_versions",
        "convai_agent_branches",
        "convai_agent_versions_deployments",
        "convai_memory_entries",
        "convai_coaching_proposals",
        "convai_templates",
        "dashboard",
        "dashboard_configuration",
        "convai_agent_drafts",
        "resource_locators",
        "assets",
        "content_generations",
        "content_templates",
        "songs",
        "transcription_tasks",
        "avatars",
        "avatar_video_generations",
        "resource_collection",
        "studio_projects",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.mcp_server_response_model import McpServerResponseModel
from ....types.mcp_tool_approval_policy import McpToolApprovalPolicy
from .raw_client import AsyncRawToolApprovalsClient, RawToolApprovalsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ToolApprovalsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawToolApprovalsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawToolApprovalsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawToolApprovalsClient
        """
        return self._raw_client

    def create(
        self,
        mcp_server_id: str,
        *,
        tool_name: str,
        tool_description: str,
        input_schema: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        approval_policy: typing.Optional[McpToolApprovalPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> McpServerResponseModel:
        """
        Add approval for a specific MCP tool when using per-tool approval mode.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        tool_name : str
            The name of the MCP tool

        tool_description : str
            The description of the MCP tool

        input_schema : typing.Optional[typing.Dict[str, typing.Any]]
            The input schema of the MCP tool (the schema defined on the MCP server before ElevenLabs does any extra processing)

        approval_policy : typing.Optional[McpToolApprovalPolicy]
            The tool-level approval policy

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.mcp_servers.tool_approvals.create(
            mcp_server_id="mcp_server_id",
            tool_name="tool_name",
            tool_description="tool_description",
        )
        """
        _response = self._raw_client.create(
            mcp_server_id,
            tool_name=tool_name,
            tool_description=tool_description,
            input_schema=input_schema,
            approval_policy=approval_policy,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self, mcp_server_id: str, tool_name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerResponseModel:
        """
        Remove approval for a specific MCP tool when using per-tool approval mode.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        tool_name : str
            Name of the MCP tool to remove approval for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.mcp_servers.tool_approvals.delete(
            mcp_server_id="mcp_server_id",
            tool_name="tool_name",
        )
        """
        _response = self._raw_client.delete(mcp_server_id, tool_name, request_options=request_options)
        return _response.data


class AsyncToolApprovalsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawToolApprovalsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawToolApprovalsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawToolApprovalsClient
        """
        return self._raw_client

    async def create(
        self,
        mcp_server_id: str,
        *,
        tool_name: str,
        tool_description: str,
        input_schema: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        approval_policy: typing.Optional[McpToolApprovalPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> McpServerResponseModel:
        """
        Add approval for a specific MCP tool when using per-tool approval mode.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        tool_name : str
            The name of the MCP tool

        tool_description : str
            The description of the MCP tool

        input_schema : typing.Optional[typing.Dict[str, typing.Any]]
            The input schema of the MCP tool (the schema defined on the MCP server before ElevenLabs does any extra processing)

        approval_policy : typing.Optional[McpToolApprovalPolicy]
            The tool-level approval policy

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.mcp_servers.tool_approvals.create(
                mcp_server_id="mcp_server_id",
                tool_name="tool_name",
                tool_description="tool_description",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            mcp_server_id,
            tool_name=tool_name,
            tool_description=tool_description,
            input_schema=input_schema,
            approval_policy=approval_policy,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, mcp_server_id: str, tool_name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerResponseModel:
        """
        Remove approval for a specific MCP tool when using per-tool approval mode.

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP Server.

        tool_name : str
            Name of the MCP tool to remove approval for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.mcp_servers.tool_approvals.delete(
                mcp_server_id="mcp_server_id",
                tool_name="tool_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(mcp_server_id, tool_name, request_options=request_options)
        return _response.data



"""
Real-time speech-to-text WebSocket helpers for ElevenLabs API.

This module provides classes for streaming audio to the ElevenLabs
speech-to-text API and receiving real-time transcription results.
"""

from .connection import RealtimeConnection, RealtimeEvents
from .scribe import ScribeRealtime, AudioFormat, CommitStrategy, RealtimeAudioOptions, RealtimeUrlOptions

__all__ = [
    "RealtimeConnection",
    "RealtimeEvents",
    "ScribeRealtime",
    "AudioFormat",
    "CommitStrategy",
    "RealtimeAudioOptions",
    "RealtimeUrlOptions",
]




# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class WebhookHmacSettings(UncheckedBaseModel):
    """
    Settings for creating an HMAC-authenticated webhook
    """

    auth_type: typing.Literal["hmac"] = pydantic.Field(default="hmac")
    """
    The authentication type for this webhook
    """

    name: str = pydantic.Field()
    """
    The display name for this webhook
    """

    webhook_url: str = pydantic.Field()
    """
    The HTTPS callback URL that will be called when this webhook is triggered
    """

    request_headers: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Optional custom request headers to include with each webhook delivery
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .detailed_music_response import DetailedMusicResponse


class MultipartMusicResponse(UncheckedBaseModel):
    """
    Multipart response structure with JSON metadata and binary audio
    """

    metadata: DetailedMusicResponse = pydantic.Field()
    """
    JSON metadata about the generated audio
    """

    audio: str = pydantic.Field()
    """
    Binary audio data in the requested format
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .media_id import MediaId


class OrderMediaResponse(UncheckedBaseModel):
    media_id: MediaId = pydantic.Field()
    """
    The ID of the media file.
    """

    name: str = pydantic.Field()
    """
    The original filename of the uploaded media.
    """

    content_type: str = pydantic.Field()
    """
    The MIME type of the media file (e.g. 'video/mp4').
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The detected or declared language of the media, if available.
    """

    signed_url: str = pydantic.Field()
    """
    A time-limited URL to download the media file.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversation_history_transcript_common_model_output import ConversationHistoryTranscriptCommonModelOutput


class TestFromConversationMetadataOutput(UncheckedBaseModel):
    conversation_id: str
    agent_id: str
    branch_id: typing.Optional[str] = None
    workflow_node_id: typing.Optional[str] = None
    original_agent_reply: typing.Optional[typing.List[ConversationHistoryTranscriptCommonModelOutput]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(TestFromConversationMetadataOutput)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .position_output import PositionOutput


class WorkflowStandaloneAgentNodeModelOutput(UncheckedBaseModel):
    position: PositionOutput = pydantic.Field()
    """
    Position of the node in the workflow.
    """

    edge_order: typing.List[str] = pydantic.Field()
    """
    The ids of outgoing edges in the order they should be evaluated.
    """

    agent_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the agent to transfer the conversation to. None means transfer within the current agent.
    """

    node_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional target node ID in the destination agent's workflow. When set, the transfer starts at this node instead of the default entry node.
    """

    delay_ms: int = pydantic.Field()
    """
    Artificial delay in milliseconds applied before transferring the conversation.
    """

    transfer_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional message sent to the user before the transfer is initiated.
    """

    enable_transferred_agent_first_message: bool = pydantic.Field()
    """
    Whether to enable the transferred agent to send its configured first message after the transfer.
    """

    preserve_client_tts_overrides: bool = pydantic.Field()
    """
    Defines whether TTS client overrides should be carried over to the transferred agent.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversation_history_transcript_system_tool_result_common_model_input_result import (
    ConversationHistoryTranscriptSystemToolResultCommonModelInputResult,
)
from .dynamic_variable_update_common_model import DynamicVariableUpdateCommonModel


class ConversationHistoryTranscriptSystemToolResultCommonModelInput(UncheckedBaseModel):
    request_id: str
    tool_name: str
    result_value: str
    is_error: bool
    is_blocked: typing.Optional[bool] = None
    tool_has_been_called: bool
    tool_latency_secs: typing.Optional[float] = None
    error_type: typing.Optional[str] = None
    raw_error_message: typing.Optional[str] = None
    dynamic_variable_updates: typing.Optional[typing.List[DynamicVariableUpdateCommonModel]] = None
    type: typing.Literal["system"] = "system"
    result: typing.Optional[ConversationHistoryTranscriptSystemToolResultCommonModelInputResult] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

StartProcedureToolErrorStatus = typing.Union[typing.Literal["not_found", "invalid_name", "already_active"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .verification_attempt_response import VerificationAttemptResponse


class VoiceVerificationResponse(UncheckedBaseModel):
    requires_verification: bool = pydantic.Field()
    """
    Whether the voice requires verification.
    """

    is_verified: bool = pydantic.Field()
    """
    Whether the voice has been verified.
    """

    verification_failures: typing.List[str] = pydantic.Field()
    """
    List of verification failures.
    """

    verification_attempts_count: int = pydantic.Field()
    """
    The number of verification attempts.
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The language of the voice.
    """

    verification_attempts: typing.Optional[typing.List[VerificationAttemptResponse]] = pydantic.Field(default=None)
    """
    Number of times a verification was attempted.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from elevenlabs import AgentWorkflowRequestModel, ObjectJsonSchemaPropertyInput, ObjectJsonSchemaPropertyOutput


def test_recursive_models():
    # Should be able to import and use the recursive types without PydanticUserError
    # ObjectJsonSchemaPropertyInput has no required fields, so it can be instantiated
    ObjectJsonSchemaPropertyInput()
    
    ObjectJsonSchemaPropertyOutput()

    AgentWorkflowRequestModel()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .telephony_direction import TelephonyDirection


class ConversationHistorySipTrunkingPhoneCallModel(UncheckedBaseModel):
    direction: TelephonyDirection
    phone_number_id: str
    agent_number: str
    external_number: str
    call_id: typing.Optional[str] = None
    call_sid: str
    sip_header_dynamic_variables: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from elevenlabs import GetSpeechHistoryResponse, ElevenLabs


def test_history():
    client = ElevenLabs()
    page_size = 5
    history = client.history.list(page_size=page_size)
    assert isinstance(history, GetSpeechHistoryResponse)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.get_agent_link_response_model import GetAgentLinkResponseModel
from .raw_client import AsyncRawLinkClient, RawLinkClient


class LinkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLinkClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLinkClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLinkClient
        """
        return self._raw_client

    def get(
        self, agent_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetAgentLinkResponseModel:
        """
        Get the current link used to share the agent with others

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAgentLinkResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.agents.link.get(
            agent_id="agent_3701k3ttaq12ewp8b7qv5rfyszkz",
        )
        """
        _response = self._raw_client.get(agent_id, request_options=request_options)
        return _response.data


class AsyncLinkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLinkClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLinkClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLinkClient
        """
        return self._raw_client

    async def get(
        self, agent_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetAgentLinkResponseModel:
        """
        Get the current link used to share the agent with others

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAgentLinkResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.agents.link.get(
                agent_id="agent_3701k3ttaq12ewp8b7qv5rfyszkz",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(agent_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from .whats_app_template_document_param_details import WhatsAppTemplateDocumentParamDetails
from .whats_app_template_image_param_details import WhatsAppTemplateImageParamDetails
from .whats_app_template_location_param_details import WhatsAppTemplateLocationParamDetails


class WhatsAppTemplateHeaderComponentParamsParametersItem_Document(UncheckedBaseModel):
    type: typing.Literal["document"] = "document"
    document: WhatsAppTemplateDocumentParamDetails

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class WhatsAppTemplateHeaderComponentParamsParametersItem_Image(UncheckedBaseModel):
    type: typing.Literal["image"] = "image"
    image: WhatsAppTemplateImageParamDetails

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class WhatsAppTemplateHeaderComponentParamsParametersItem_Location(UncheckedBaseModel):
    type: typing.Literal["location"] = "location"
    location: WhatsAppTemplateLocationParamDetails

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class WhatsAppTemplateHeaderComponentParamsParametersItem_Text(UncheckedBaseModel):
    type: typing.Literal["text"] = "text"
    parameter_name: typing.Optional[str] = None
    text: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


WhatsAppTemplateHeaderComponentParamsParametersItem = typing_extensions.Annotated[
    typing.Union[
        WhatsAppTemplateHeaderComponentParamsParametersItem_Document,
        WhatsAppTemplateHeaderComponentParamsParametersItem_Image,
        WhatsAppTemplateHeaderComponentParamsParametersItem_Location,
        WhatsAppTemplateHeaderComponentParamsParametersItem_Text,
    ],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

ClipAnimationEnterEffect = typing.Union[
    typing.Literal[
        "none",
        "fade",
        "float",
        "gentle_float",
        "zoom_in",
        "drop",
        "slide_left",
        "slide_right",
        "slide_up",
        "slide_down",
        "pop",
        "bounce",
        "spin",
        "slide_bounce",
    ],
    typing.Any,
]
